/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.text;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.text.ChoiceFormat;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon.Rank;
import static org.openide.util.NbBundle.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class TaxonCountFormat
  {
    private static final Class<TaxonCountFormat> _ = TaxonCountFormat.class;
    
    protected int initializationCount = 0; // for testing

    @CheckForNull
    protected Locale locale;

    @Nonnull
    private final Rank rank;

    private ChoiceFormat choice;

    public TaxonCountFormat (final @Nonnull Rank rank)
      {
        this.rank = rank;
      }

    @Nonnull
    public String format (final int n)
      {
        initialize();
        return String.format(choice.format(n), n);
      }

    private synchronized  void initialize()
      {
        if ((locale == null) || !locale.equals(Locale.getDefault()))
          {
            initializationCount++;
            locale = Locale.getDefault();
            final Map<Integer, String> maleQuantifierMap = new HashMap<Integer, String>();
            final Map<Integer, String> femaleQuantifierMap = new HashMap<Integer, String>();
            final Map<Rank, String> singularMap = new HashMap<Rank, String>();
            final Map<Rank, String> pluralMap = new HashMap<Rank, String>();
            Map<Rank, String> cardinalityForZero = null;

            maleQuantifierMap.put(0, getMessage(_, "maleNone"));
            maleQuantifierMap.put(1, getMessage(_, "maleOne"));
            femaleQuantifierMap.put(0, getMessage(_, "femaleNone"));
            femaleQuantifierMap.put(1, getMessage(_, "femaleOne"));

            singularMap.put(Rank.CLASS, getMessage(_, "class"));
            pluralMap.  put(Rank.CLASS, getMessage(_, "classes"));
            singularMap.put(Rank.FAMILY, getMessage(_, "family"));
            pluralMap.  put(Rank.FAMILY, getMessage(_, "families"));
            singularMap.put(Rank.GENUS, getMessage(_, "genus"));
            pluralMap.  put(Rank.GENUS, getMessage(_, "genera"));
            singularMap.put(Rank.ORDER, getMessage(_, "order"));
            pluralMap.  put(Rank.ORDER, getMessage(_, "orders"));
            singularMap.put(Rank.SPECIES, getMessage(_, "species"));
            pluralMap.  put(Rank.SPECIES, getMessage(_, "speciesPlural"));
            singularMap.put(Rank.SUBSPECIES, getMessage(_, "subspecies"));
            pluralMap.  put(Rank.SUBSPECIES, getMessage(_, "subspeciesPlural"));

            if (locale.getLanguage().equals("it"))
              {
                cardinalityForZero = singularMap;
              }

            else // english or fallback
              {
                cardinalityForZero = pluralMap;
              }

            final double[] limits = { 0, 1, 2 };

            final Map<Integer, String> quantifierMap = rank.equals(Rank.GENUS) || rank.equals(Rank.ORDER) ?
                maleQuantifierMap : femaleQuantifierMap;
            
            final String[] parts =
              {
                quantifierMap.get(0) + " " + cardinalityForZero.get(rank),
                quantifierMap.get(1) + " " + singularMap.get(rank),
                "%d " + pluralMap.get(rank)
              };

            choice = new ChoiceFormat(limits, parts);
          }
      }
  }
